/// @file   Adapter.h
/// @brief  Dummy Plugin concrete factories.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef ADAPTER_H
#define	ADAPTER_H

#include <dmlite/cpp/authn.h>
#include <dmlite/cpp/catalog.h>
#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/inode.h>
#include <dmlite/cpp/poolmanager.h>
#include <dmlite/cpp/utils/poolcontainer.h>

namespace dmlite {


  /// Fake Factory for Pool resources
  class IntConnectionFactory: public PoolElementFactory<int> {
  public:
    IntConnectionFactory() {};
    virtual ~IntConnectionFactory() {};

    int create() { return 1; }
    void   destroy(int) {};
    bool   isValid(int) { return true; }

  protected:
  private:
  };



  /// Concrete factory for DPNS/LFC wrapper
  class NsAdapterFactory: public CatalogFactory, public INodeFactory, public AuthnFactory {
  public:
    /// Constructor
    NsAdapterFactory() throw (DmException);
    /// Destructor
    ~NsAdapterFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);

    INode*   createINode  (PluginManager*)  throw (DmException);
    Catalog* createCatalog(PluginManager*)  throw (DmException);
    Authn*   createAuthn  (PluginManager*)  throw (DmException);

    PoolContainer<int> *getPool() {return &connectionPool_;}
  protected:
    unsigned retryLimit_;
    bool hostDnIsRoot_;
    std::string hostDn_;
    std::string dpnsHost_;

    /// Connection pool.
    PoolContainer<int> connectionPool_;
    IntConnectionFactory connectionFactory_;
  };

  /// Concrete factory for DPM wrapper
  class DpmAdapterFactory: public NsAdapterFactory, public PoolManagerFactory, public PoolDriverFactory {
  public:
    /// Constructor
    DpmAdapterFactory() throw (DmException);
    /// Destructor
    ~DpmAdapterFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);

    Catalog*     createCatalog(PluginManager*)     throw (DmException);
    PoolManager* createPoolManager(PluginManager*) throw (DmException);

    std::string implementedPool() throw();
    PoolDriver* createPoolDriver(void) throw (DmException);

    PoolContainer<int> *getPool() {return &connectionPool_;}
  protected:
    unsigned retryLimit_;


    /// Connection pool.
    PoolContainer<int> connectionPool_;
    IntConnectionFactory connectionFactory_;

    std::string tokenPasswd_;
    bool        tokenUseIp_;
    unsigned    tokenLife_;

    /// Admin username for replication.
    std::string adminUsername_;
  };

};

#endif	// ADAPTER_H

